'use strict';

module.exports = {

    initDragAndDrop: function() {

        // document.addEventListener('mousemove', dropIf, false);
        document.addEventListener('dragover', this.dragListener, false);
        // document.addEventListener('touchmove', dragListener, false);

        this.initPlaceholderList();

        this.sortableList = document.getElementById(this.selector);

        this.sortableList.querySelectorAll('li').forEach(el => {
            el.setAttribute('draggable', 'true');

            el.addEventListener('dragstart', this.onDragStart, false);
            // el.addEventListener('touchstart', onDragStart, false);

            el.addEventListener('dragenter', this.onDragEnter, false);
            // el.addEventListener('dragexit', removeStyles, false);
            el.addEventListener('dragleave', this.onDragLeave, false);
            el.addEventListener('dragend', this.onDragEnd, false);
            el.addEventListener('drop', this.onDrop, false);
        });
    },

    onDragStart: function(e) {
        this.draggedNode = e.target;
        this.draggedNode.classList.add('dragged');
        e.dataTransfer.setData('text', 'Drag started!'); // Hack for Firefox!
    },

    onDragEnter: function(e) {
        // console.log(e.target);
        e.preventDefault();

        // if (cannotDropOn(e.target)) {
        //     return;
        // }

        if (['LI', 'UL'].indexOf(e.target.nodeName) > -1) {
            this.targetedNode = e.target;
        }
    },

    dragListener: function(e) {
        this.updateCoordinations(e);
        this.managePlaceholderLists(e);
        this.dropIf(e);
    },

    updateCoordinations: function(e) {
        this.calcMouseCoords(e);
        this.calcMouseToTargetedElDist();
    },

    dropIf: function(e) {
        if (!this.canBeDropped()) {
            return;
        }

        if (this.targetedNode.nodeName === 'LI' && !this.cursorIsIndentedEnough()) {
            if (
                this.distances.mouseTo.targetedElTop < 0
                && this.distances.mouseTo.targetedElTop > this.distances.droppingEdgeNegative
                // && mouseHasMovedUp()
            ) {
                this.dropTheItem('before');
            } else if (
                this.distances.mouseTo.targetedElBot < 0
                && this.distances.mouseTo.targetedElBot > this.distances.droppingEdgeNegative
                // && mouseHasMovedDown()
            ) {
                this.dropTheItem('after');
            }
        } else if (this.targetedNode.nodeName === 'UL') {
            this.dropTheItem('inside');
        }
    },

    dropTheItem: function(place) {
        switch (place) {
            case 'before':
                this.targetedNode.parentNode.insertBefore(this.draggedNode, this.targetedNode);
                break;
            case 'after':
                this.insertAfter(this.draggedNode, this.targetedNode);
                break;
            case 'inside':
                this.targetedNode.appendChild(this.draggedNode);
                break;
        }

        this.draggedNode.dispatchEvent(this.dropEvent);
    },

    calcMouseCoords: function(e) {
        // cursorX = e.screenX;
        // cursorY = e.screenY;

        // we're having the client coords because on the next lines, we use getBoundingClientRect
        // which behaves in the same way
        this.cursorX = e.clientX;
        this.cursorY = e.clientY;
        // console.log('X: ' + cursorX + ', Y: ' + cursorY);
    },

    calcMouseToTargetedElDist: function() {
        if (!this.targetedNode) {
            return;
        }

        let offset = this.targetedNode.getBoundingClientRect();
        this.targetedNodeX = offset.left;
        this.targetedNodeY = offset.top;

        let result = this.targetedNodeY - this.cursorY;
        this.distances.mouseTo.targetedElTop = result;
        this.distances.mouseTo.targetedElTopAbs = Math.abs(result);
        this.dimensions.targetedEl.H = this.targetedNode.clientHeight;
        this.distances.mouseTo.targetedElBot = this.distances.mouseTo.targetedElTopAbs - this.dimensions.targetedEl.H;
    },

    onDragLeave: function(e) {
        e.preventDefault();
    },

    onDragEnd: function(e) {
        e.preventDefault();
        this.draggedNode.classList.remove('dragged');
        this.cleanupPlaceholderLists();
    },

    areNested: function(child, parent) {
        let isChild = false;
        parent.querySelectorAll('li').forEach(li => {
            if (li === child) {
                isChild = true;
            }
        });
        return isChild;
    },

    cursorIsIndentedEnough: function() {
        return this.cursorX - this.targetedNodeX > 50;
    },

    mouseHasMovedUp: function() {
        // console.log('mouse Has Moved UP');
        return this.draggedNode.getBoundingClientRect().top > this.cursorY;
    },

    mouseHasMovedDown: function() {
        // console.log('mouse Has Moved DOWN');
        return !this.mouseHasMovedUp();
    },

    mouseIsTooCloseToTop: function() {
        return this.cursorY - this.targetedNodeY < this.distances.droppingEdge;
    },

    insertAfter: function(newNode, referenceNode) {
        referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
    },

    managePlaceholderLists: function(e) {

        let actions = this.analysePlaceHolderSituation(e);

        actions.forEach(action => {
            switch (action) {
                case 'add':
                    this.cleanupPlaceholderLists();
                    this.addPlaceholderList();
                    break;
                case 'cleanup':
                    this.cleanupPlaceholderLists();
                    break;
                default:
                    return;
                    break;
            }
        });
    },

    targetedNodeIsPlaceholder: function() {
        return this.targetedNode.nodeName === 'UL' && this.targetedNode.classList.contains('placeholder');
    },

    analysePlaceHolderSituation: function(e) {
        if (!this.targetedNode || this.areNested(this.targetedNode, this.draggedNode)) {
            return [];
        }

        let actions = [];

        if (!this.cursorIsIndentedEnough() || this.mouseIsTooCloseToTop()) {
            if (!this.targetedNodeIsPlaceholder()) {
                actions.push('cleanup');
            }
        } else if (this.targetedNode !== this.draggedNode
            && this.targetedNode.nodeName === 'LI'
            && !this.targetedNode.querySelectorAll('ul').length) {
            actions.push('add');
        }

        return actions;
    },

    addPlaceholderList: function() {
        this.targetedNode.appendChild(this.getPlaceholderList());
    },

    canBeDropped: function() {
        let result = true;

        result &= !!this.targetedNode && this.targetedNode !== this.draggedNode;
        result &= !(this.targetedNode.nodeName === 'UL' && this.targetedNode.querySelectorAll('li').length);
        result &= !this.areNested(this.targetedNode, this.draggedNode);

        return result;
    },

    cleanupPlaceholderLists: function() {
        this.sortableList.querySelectorAll('ul').forEach(ul => {
            if (!ul.querySelectorAll('li').length) {
                ul.remove();
            } else if (ul.classList.contains('placeholder')) {
                ul.classList.remove('placeholder');
            }
        });
    },

    initPlaceholderList: function() {
        this.placeholderUl = document.createElement('ul');
        this.placeholderUl.classList.add("placeholder");
    },

    getPlaceholderList: function() {
        this.placeholderInUse = this.placeholderUl.cloneNode(true);
        return this.placeholderInUse;
    },

    onDrop: function() {
        this.cleanupPlaceholderLists();
    },

    nestedSortable: function(options) {
        let defaults = {
            droppingEdge: 15
        };

        // Set default options
        for (let name in defaults) {
            !(name in options) && (options[name] = defaults[name]);
        }

        this.selector = options.el;
        this.distances.droppingEdge = options.droppingEdge;
        this.distances.droppingEdgeNegative = options.droppingEdge * -1;

        this.initDragAndDrop();
    },

    selector: '',

    sortableList: null,
    placeholderUl: null,
    placeholderInUse: null,
    draggedNode: null,
    targetedNode: null,

    distances: {
        mouseTo: {
            targetedElTop: undefined
        }
    },
    dimensions: {
        targetedEl: {
            H: undefined
        }
    },
    cursorX: null,
    cursorY: null,
    targetedNodeX: null,
    targetedNodeY: null,

    dropEvent: new Event('drop')
};
